@@ -5,4 +5,7 @@
 
 mod stack;
 
-pub use self::stack::{StackExecutor, MemoryStackState, StackState, StackSubstateMetadata, StackExitKind, PrecompileOutput};
+pub use self::stack::{
+	MemoryStackState, PrecompileFn, PrecompileOutput, StackExecutor, StackExitKind, StackState,
+	StackSubstateMetadata,
+};
