@@ -2,14 +2,13 @@
 
 #![deny(warnings)]
 #![forbid(unsafe_code, unused_variables)]
-
 #![cfg_attr(not(feature = "std"), no_std)]
 
 extern crate alloc;
 
 pub use evm_core::*;
-pub use evm_runtime::*;
 pub use evm_gasometer as gasometer;
+pub use evm_runtime::*;
 
 #[cfg(feature = "tracing")]
 pub mod tracing;
@@ -19,13 +18,13 @@ macro_rules! event {
 	($x:expr) => {
 		use crate::tracing::Event::*;
 		$x.emit();
-	}
+	};
 }
 
 #[cfg(not(feature = "tracing"))]
 macro_rules! event {
-	($x:expr) => { }
+	($x:expr) => {};
 }
 
-pub mod executor;
 pub mod backend;
+pub mod executor;
